import json
import os
import unittest

from dotenv import load_dotenv

from trading212 import client


class TestAccountData(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        token = os.getenv("TOKEN")
        self.client = client.Client(token, demo=True)

    def test_get_account_cash(self):
        response = self.client.get_account_cash()

        self.assertIn("free", response)
        self.assertIsInstance(response["free"], float)
        self.assertIn("total", response)
        self.assertIsInstance(response["total"], float)
        self.assertIn("result", response)
        self.assertIsInstance(response["result"], float)

    def test_get_account_metadata(self):
        response = self.client.get_account_metadata()

        self.assertIn("id", response)
        self.assertIsInstance(response["id"], int)
        self.assertIn("currencyCode", response)
        self.assertIsInstance(response["currencyCode"], str)



import datetime
import json
import os
import unittest

from trading212 import client
from dotenv import load_dotenv

HELPER_PIE = {
    "name": "Test Pie",
    "icon": "Unicorn",
    "instrumentShares": {
        "RPIl_EQ": 1.0
    },
    "goal": 1000,
    "dividendCashAction": "REINVEST",
    "endDate": "2030-09-10T10:00:00Z"
}


class TestPies(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        token = os.getenv("TOKEN")
        self.client = client.Client(token, demo=True)

    def assert_helper_pie(self, pie):
        """
        Helper function to assert the properties of the helper pie.
        This is used to ensure that the pie has the same properties as the helper pie created.
        """
        self.assertIsInstance(pie, dict)

        self.assertIn('settings', pie)
        self.assertIn('instruments', pie)

        self.assertEqual(pie['settings']['name'], HELPER_PIE['name'])
        self.assertEqual(pie['settings']['icon'], HELPER_PIE['icon'])
        self.assertEqual(pie['instruments'][0]['ticker'], 'RPIl_EQ')
        self.assertEqual(pie['instruments'][0]['expectedShare'], HELPER_PIE['instrumentShares']['RPIl_EQ'])
        self.assertEqual(pie['settings']['goal'], HELPER_PIE['goal'])
        self.assertEqual(pie['settings']['dividendCashAction'], HELPER_PIE['dividendCashAction'])

        # The API does not return the endDate in the same format as the helper pie. Nor does it return the correct time.
        helper_time = datetime.datetime.strptime(HELPER_PIE["endDate"], '%Y-%m-%dT%H:%M:%SZ')
        api_time = datetime.datetime.strptime(pie['settings']['endDate'], '%Y-%m-%dT%H:%M:%S.%f%z')
        self.assertEqual(helper_time.strftime('%Y-%m-%d'), api_time.strftime('%Y-%m-%d'))

    def test_create_pie(self):
        response = self.client.create_pie(HELPER_PIE)

        self.assert_helper_pie(response)

        self.client.delete_pie(response['settings']['id'])

    def test_get_pies(self):
        self.client.create_pie(HELPER_PIE)
        pies = self.client.get_pies()

        self.assertIsInstance(pies, list)

        self.client.delete_pie(pies[0]['id'])

    def test_get_pie(self):
        pie_id = self.client.create_pie(HELPER_PIE)['settings']['id']
        pie = self.client.get_pie(pie_id)

        self.assert_helper_pie(pie)

        self.client.delete_pie(pie_id)

    def test_update_pie(self):
        pie = self.client.create_pie(HELPER_PIE)
        new_pie = {
            "name": "Updated Pie",
            "instrumentShares": {
                "RPIl_EQ": 0.5,
                "AAPL_US_EQ": 0.5
            },
            "goal": 2000,
        }
        updated_pie = self.client.update_pie(pie['settings']['id'], new_pie)

        print(json.dumps(updated_pie, indent=4))

        self.assertEqual(updated_pie['settings']['name'],
                         new_pie['name'])
        self.assertEqual(updated_pie['instruments'][0]['expectedShare'],
                         new_pie['instrumentShares']['RPIl_EQ'])
        self.assertEqual(updated_pie['instruments'][1]['expectedShare'],
                         new_pie['instrumentShares']['AAPL_US_EQ'])
        self.assertEqual(updated_pie['settings']['goal'],
                         new_pie['goal'])

        self.client.delete_pie(pie['settings']['id'])

    def test_duplicate_pie(self):
        pie = self.client.create_pie(HELPER_PIE)
        new_pie = {
            "name": "Duplicated Pie"
        }
        duplicated_pie = self.client.duplicate_pie(pie['settings']['id'], new_pie)

        self.assertEqual(new_pie['name'], duplicated_pie['settings']['name'])

        self.client.delete_pie(pie['settings']['id'])
        self.client.delete_pie(duplicated_pie['settings']['id'])

    def test_delete_pie(self):
        pie = self.client.create_pie(HELPER_PIE)
        response = self.client.delete_pie(pie['settings']['id'])

        self.assertIsNone(response)



import os
import unittest

from trading212 import client
from dotenv import load_dotenv


class TestInstrumentMetadata(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        token = os.getenv("TOKEN")
        self.client = client.Client(token, demo=True)

    def test_get_exchanges(self):
        exchanges = self.client.get_exchanges()
        self.assertIsInstance(exchanges, list)

    def test_get_instruments(self):
        instruments = self.client.get_instruments()
        self.assertIsInstance(instruments, list)




import os
import unittest

from trading212 import client
from dotenv import load_dotenv


class TestClient(unittest.TestCase):

    def setUp(self):
        load_dotenv()
        token = os.getenv("TOKEN")
        self.client = client.Client(token, demo=True)

    def test_init(self):
        # Test with default environment (live)
        live_client = client.Client(token="test_token")
        self.assertEqual(live_client.base_url, "https://live.trading212.com/api/v0/")
        self.assertEqual(live_client.headers['Authorization'], "test_token")
        self.assertEqual(live_client.headers['Content-Type'], 'application/json')

    def test_init_live(self):
        # Test with live environment
        live_client = client.Client(token="test_token", demo=False)
        self.assertEqual(live_client.base_url, "https://live.trading212.com/api/v0/")
        self.assertEqual(live_client.headers['Authorization'], "test_token")
        self.assertEqual(live_client.headers['Content-Type'], 'application/json')

    def test_init_demo(self):
        # Test with demo environment
        demo_client = client.Client(token="test_token", demo=True)
        self.assertEqual(demo_client.base_url, "https://demo.trading212.com/api/v0/")
        self.assertEqual(demo_client.headers['Authorization'], "test_token")
        self.assertEqual(demo_client.headers['Content-Type'], 'application/json')




import time

import requests

API_VERSION = "v0"

LIVE_API_URL = "https://live.trading212.com/api/{}/".format(API_VERSION)
DEMO_API_URL = "https://demo.trading212.com/api/{}/".format(API_VERSION)


class Client:

    def __init__(self, token: str, demo: bool = False):
        self.base_url = DEMO_API_URL if demo else LIVE_API_URL
        self.headers = {
            'Authorization': token,
            'Content-Type': 'application/json'
        }

    def make_backoff_request(self, url, method='GET', data=None, backoff=30):
        """
        Make a request with a backoff timer for rate limiting.
        :param url: The URL to request.
        :param method: The HTTP method to use (default is GET).
        :param data: The data to send with the request (for POST/PUT).
        :param backoff: The number of seconds to wait before retrying on rate limit.
        :return: The response from the request.
        """
        response = requests.request(method, url, headers=self.headers, json=data)
        if response.status_code == 429:
            print(f"Rate limit exceeded. Retrying after {backoff} seconds...")
            time.sleep(backoff)
            response = requests.request(method, url, headers=self.headers, json=data)
        response.raise_for_status()

        if len(response.text) != 0:
            return response.json()
        return None

    def get_exchanges(self):
        """
        Get a list of available exchanges.
        https://t212public-api-docs.redoc.ly/#operation/exchanges
        :return: A list of exchanges.
        """
        return self.make_backoff_request(
            self.base_url + "equity/metadata/exchanges",
            backoff=30
        )

    def get_instruments(self):
        """
        Get a list of instruments.
        https://t212public-api-docs.redoc.ly/#operation/instruments
        :return: A list of instruments.
        """
        return self.make_backoff_request(
            self.base_url + "equity/metadata/instruments",
            backoff=30
        )

    def get_pies(self):
        """
        Get a list of pies.
        https://t212public-api-docs.redoc.ly/#operation/pies
        :return: A list of pies.
        """
        return self.make_backoff_request(
            self.base_url + "equity/pies",
            backoff=30
        )

    def create_pie(self, pie):
        """
        Create a new pie.
        https://t212public-api-docs.redoc.ly/#operation/create
        :param pie: JSON object representing pie to create. See the API documentation for the required fields.
        :return: The created pie.
        """
        return self.make_backoff_request(
            self.base_url + "equity/pies",
            method='POST',
            data=pie,
            backoff=5
        )

    def get_pie(self, pie_id):
        """
        Get a specific pie by its ID.
        https://t212public-api-docs.redoc.ly/#operation/getDetailed
        :param pie_id: The ID of the pie to retrieve.
        :return: The pie with the specified ID.
        """
        return self.make_backoff_request(
            self.base_url + "equity/pies/{}".format(pie_id),
            backoff=5
        )

    def update_pie(self, pie_id, updated_pie):
        """
        Update an existing pie.
        https://t212public-api-docs.redoc.ly/#operation/update
        :param pie_id: The ID of the pie to update.
        :param updated_pie: JSON object representing the updated pie. See the API documentation for the required fields.
        :return: The updated pie.
        """
        return self.make_backoff_request(
            self.base_url + "equity/pies/{}".format(pie_id),
            method='POST',
            data=updated_pie,
            backoff=5
        )

    def duplicate_pie(self, pie_id, duplicated_pie):
        """
        Duplicate an existing pie.
        https://t212public-api-docs.redoc.ly/#operation/duplicatePie
        :param pie_id: The ID of the pie to duplicate.
        :param duplicated_pie: JSON object representing the fields to change. Must include the 'name' field.
        :return: The duplicated pie.
        """
        return self.make_backoff_request(
            self.base_url + "equity/pies/{}/duplicate".format(pie_id),
            method='POST',
            data=duplicated_pie,
            backoff=5
        )

    def delete_pie(self, pie_id):
        """
        Delete a specific pie by its ID.
        https://t212public-api-docs.redoc.ly/#operation/delete
        :param pie_id: The ID of the pie to delete.
        :return: None if successful, raises HTTPError if the request fails.
        """
        return self.make_backoff_request(
            self.base_url + "equity/pies/{}".format(pie_id),
            method='DELETE',
            backoff=5
        )

    def get_account_cash(self):
        """
        Get the cash balance of the account.
        https://t212public-api-docs.redoc.ly/#operation/accountCash
        :return: The cash balance as a float.
        """
        return self.make_backoff_request(
            self.base_url + "equity/account/cash",
            backoff=2
        )

    def get_account_metadata(self):
        """
        Get metadata about the account.
        https://t212public-api-docs.redoc.ly/#operation/accountMetadata
        :return: A dictionary containing account metadata.
        """
        return self.make_backoff_request(
            self.base_url + "equity/account/info",
            backoff=2
        )
